package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class DistributionCenter extends AbstractBehavior<DistributionCenter.Message> {

    private final ActorRef<AddressBook.Message> addressBook;
    private final List<Paket> lagerraum = new ArrayList<>();

    private DistributionCenter(ActorContext<Message> context, ActorRef<AddressBook.Message> addressBook) {
        super(context);
        this.addressBook = addressBook;

        // Erhalten Sie die replyTo-Referenz aus dem Kontext
        ActorRef<List<ActorRef<Customer.Message>>> replyTo = context.spawnAnonymous(Behaviors.<List<ActorRef<Customer.Message>>>receive((ctx, message) -> {
            if (message instanceof List) {
                @SuppressWarnings("unchecked")
                List<ActorRef<Customer.Message>> customers = (List<ActorRef<Customer.Message>>) message;
                ctx.getLog().info("Kundenliste erhalten: {}", customers);
                // Nachricht senden, um Routen zu generieren
                context.getSelf().tell(new GenerateRoutes(customers));
                return Behaviors.same();
            }
            return Behaviors.unhandled();
        }));

        // Anfrage an das AddressBook senden, um die Kundenliste abzurufen
        addressBook.tell(new AddressBook.GetCustomers(replyTo));
    }

    public static Behavior<Message> create(ActorRef<AddressBook.Message> addressBook) {
        return Behaviors.setup(context -> new DistributionCenter(context, addressBook));
    }

    interface Message {}

    public static class ArriveMessage implements Message {
        private final List<Paket> pakete;
        private final ActorRef<DeliveryCar.Message> truck;

        public ArriveMessage(List<Paket> pakete, ActorRef<DeliveryCar.Message> truck) {
            this.pakete = pakete;
            this.truck = truck;
        }

        public List<Paket> getPakete() {
            return pakete;
        }

        public ActorRef<DeliveryCar.Message> getTruck() {
            return truck;
        }
    }

    public static class LoadMessage implements Message {
        private final List<Paket> pakete;

        public LoadMessage(List<Paket> pakete) {
            this.pakete = pakete;
        }

        public List<Paket> getPakete() {
            return pakete;
        }
    }

    public static class GenerateRoutes implements Message {
        private final List<ActorRef<Customer.Message>> customers;

        public GenerateRoutes(List<ActorRef<Customer.Message>> customers) {
            this.customers = customers;
        }

        public List<ActorRef<Customer.Message>> getCustomers() {
            return customers;
        }
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(ArriveMessage.class, this::onArriveMessage)
                .onMessage(LoadMessage.class, this::onLoadMessage)
                .onMessage(GenerateRoutes.class, this::onGenerateRoutes)
                .build();
    }

    private Behavior<Message> onArriveMessage(ArriveMessage msg) {
        // Füge alle Pakete aus der Ankunftsnachricht dem Lagerraum hinzu
        List<Paket> arrivedPackages = msg.getPakete();
        lagerraum.addAll(arrivedPackages);
        getContext().getLog().info("Angekommene Pakete: {}", arrivedPackages);

        // Entferne zufällig 3 Pakete aus dem Lagerraum und sende sie dem angekommenen Wagen
        int numPackagesToSend = Math.min(3, lagerraum.size());
        List<Paket> packagesToSend = new ArrayList<>();
        for (int i = 0; i < numPackagesToSend; i++) {
            packagesToSend.add(lagerraum.remove(0));
        }
        msg.getTruck().tell(new DeliveryCar.LoadMessage(packagesToSend));

        // Ausgabe, wie viele Pakete nun im Verteilzentrum vorhanden sind
        getContext().getLog().info("Pakete im Verteilzentrum nach Laden: {}", lagerraum.size());

        return this;
    }

    private Behavior<Message> onLoadMessage(LoadMessage msg) {
        // Ausgabe, wie viele Pakete nun im Verteilzentrum vorhanden sind
        getContext().getLog().info("Pakete im Verteilzentrum nach Laden: {}", lagerraum.size());

        return this;
    }

    private Behavior<Message> onGenerateRoutes(GenerateRoutes msg) {
        List<ActorRef<Customer.Message>> customers = msg.getCustomers();
        generateRoutes(customers);
        return this;
    }

    private void generateRoutes(List<ActorRef<Customer.Message>> customers) {
        // Überprüfen, ob die Kundenliste null oder leer ist
        if (customers == null || customers.isEmpty()) {
            getContext().getLog().info("Kundenliste ist leer oder null. Keine Routen generiert.");
            return;
        }

        // Erstelle vier Lastwagen mit zufälligen Routen
        for (int i = 0; i < 4; i++) {
            List<ActorRef<Customer.Message>> route = generateRandomRoute(customers);
            getContext().spawn(DeliveryCar.create(route), "deliveryCar" + i);
            getContext().getLog().info("Lieferwagen {} erstellt mit Route: {}", i, route);
        }
    }

    // Dummy-Methode zum Generieren einer zufälligen Route
    private List<ActorRef<Customer.Message>> generateRandomRoute(List<ActorRef<Customer.Message>> customers) {
        List<ActorRef<Customer.Message>> route = new ArrayList<>(customers);
        Collections.shuffle(route);
        return route;
    }
}
